/***************************************************************************
                          commandingfifo.cpp  -  description
                             -------------------
    begin                : Thu Jun 1 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivan_m_r@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "commandingfifo.h"

#include <strstream>
#include "dynamicsystem.h"

CCommandingFifo::CCommandingFifo()
{
}
CCommandingFifo::~CCommandingFifo()
{
}
/**  */
void CCommandingFifo::Arrange(void* ExtraParameterL = NULL)
{
	SetMode(CRTFifo::WRITE);
	SetIdentifier("commanding_fifo2");
}
/**  */
void CCommandingFifo::GenerateCode(ostream& StreamL, const CCodeGenerator* CallerPL,
	const string& SectionL, void* ExtraParameterL = NULL) const
{
	if (CallerPL->DescendantOf("CDynamicSystem"))
	{
		if (SectionL == "fifos declaration")
			GenerateDeclaration(StreamL, ExtraParameterL);
		else
			if (SectionL == "functions declaration")
				GenerateFunctions(StreamL, ExtraParameterL);
			else	
				if (SectionL == "fifos initialization")
					GenerateInitialization(StreamL, ExtraParameterL);
				else
					if (SectionL == "before tasks wakeup")
						GenerateBeforeTasksWakeUp(StreamL, ExtraParameterL);
					else
						if (SectionL == "fifos destruction")
							GenerateDestruction(StreamL, ExtraParameterL);	
	}
}
/**  */
string CCommandingFifo::Class() const
{
	return "CCommandingFifo";
}
/**  */
bool CCommandingFifo::DescendantOf(const string& ClassNameL) const
{
	if (ClassNameL == "CCommandingFifo")
		return true;
	else
		return CRTFifo::DescendantOf(ClassNameL);
}
/**  */
void CCommandingFifo::GenerateIncludes(CStringsSet& StringsSetL,
	void* ExtraParameterL = NULL) const
{
	StringsSetL.insert("#include <unistd.h>");
	StringsSetL.insert("#include <rtl_fifo.h>");
	StringsSetL.insert("#include <rtl_time.h>");
	StringsSetL.insert("#include <sys/types.h>");
	StringsSetL.insert("#include <sys/stat.h>");
	// StringsSetL.insert("#include <fcntl.h>");		Produces a parse error in such file
}
/**  */
//Del by KDevelop: void CCommandingFifo::GenerateUnrepeatedCode(CStringsSet& StringsSetL,
//Del by KDevelop: 	const CCodeGenerator* CallerPL, const string& SectionL,
//Del by KDevelop: 	void* ExtraParameterL = NULL) const
//Del by KDevelop: {
//Del by KDevelop: 	Components().GenerateUnrepeatedCode(StringsSetL, CallerPL, SectionL);
//Del by KDevelop:
//Del by KDevelop: 	if (CallerPL->DescendantOf("CDynamicSystem"))
//Del by KDevelop: 		if (SectionL == "global variables declaration")
//Del by KDevelop: 			GenerateGlobalVariable(StringsSetL, ExtraParameterL);
//Del by KDevelop: }
/**  */
//Del by KDevelop: void CCommandingFifo::GenerateGlobalVariable(CStringsSet& StringsSetL,
//Del by KDevelop: 	void* ExtraParameterL = NULL) const
//Del by KDevelop: {
//Del by KDevelop: 	StringsSetL.insert("hrtime_t start_time;");
//Del by KDevelop: 	StringsSetL.insert("char run;");
//Del by KDevelop: }
/**  */
const CDynamicSystem* CCommandingFifo::Container() const
{
	return (CDynamicSystem*)CRTFifo::Container();
}
/**  */
void CCommandingFifo::GenerateInitialization(ostream& Stream,
	void* pExtraParameter = NULL) const
{
	Stream << "  rtf_destroy(" << Number() << ");" << endl;
	Stream << "  rtf_create(" << Number() << ", " << Size() << ");" << endl;
	Stream << "  rtf_create_handler(" << Number() << ", &commanding_fifo_handler);" << endl;
	Stream << "  " << Identifier() << " = open(\"/dev/rtf" << Number() <<
		"\", O_NONBLOCK + O_RDONLY);" << endl;
}
/**  */
void CCommandingFifo::GenerateFunctions(ostream& StreamL,
	void* ExtraParameterL = NULL) const
{
	StreamL << "int commanding_fifo_handler(unsigned int fifo)";
	StreamL << "{" << endl;
	StreamL << "  static int command;" << endl;
	StreamL << "  read(" << Identifier() << ", &command, sizeof(command));" << endl;
	StreamL << "  switch (command)" << endl;
	StreamL << "  {" << endl;
  StreamL << "    case 0:" << endl;
  StreamL << "      stop_execution();" << endl;
  StreamL << "      break;" << endl;
  StreamL << "    case 1:" << endl;
	StreamL << "      start_execution();" << endl;
  StreamL << "      break;" << endl;
	StreamL << "  };" << endl;
	StreamL << "  return 0;" << endl;
	StreamL << "}" << endl;
}
/**  */
void CCommandingFifo::GenerateBeforeTasksWakeUp(ostream& StreamL,
	void* ExtraParameterL = NULL) const
{
	StreamL << "  return 0;" << endl;
}
/**  */
void CCommandingFifo::GenerateDestruction(ostream& StreamL,
	void* ExtraParameterL = NULL) const
{
	StreamL << "  rtf_destroy(" << Number() << ");" << endl;
}
/**  */
void CCommandingFifo::GenerateCode(CStringsSet& StringsSetL,
	const CCodeGenerator* CallerPL, const string& SectionL,
	void* ExtraParameterL = NULL) const
{
	if (CallerPL->DescendantOf("CDynamicSystem"))
		if (SectionL == "includes")
			GenerateIncludes(StringsSetL, ExtraParameterL);
}
/**  */
bool CCommandingFifo::SetNumber(unsigned int uNumber)
{
  strstream Stream;

  Stream << uNumber << ends;
	CRTFifo::SetNumber(Stream.str());

	if (uNumber >= 0)
		return true;
	else
	{
		AddError(400);
		return false;
	}
}
/**  */
unsigned int CCommandingFifo::Number() const
{
	strstream Stream;
	long long Number;

	Stream << CRTFifo::Number();
	Stream >> Number;
	return Number;
}
/**  */
bool CCommandingFifo::SetSize(int iSize)
{
  strstream Stream;

  Stream << iSize << ends;
	CRTFifo::SetSize(Stream.str());

	if (iSize > 0)
		return true;
	else
	{
		AddError(401);
		return false;
	}
}
/**  */
int CCommandingFifo::Size() const
{
	strstream Stream;
	int iSize;

	Stream << CRTFifo::Size();
	Stream >> iSize;
	return iSize;
}
/**  */
unsigned int CCommandingFifo::MaximumInComponents() const
{
	return 1;
}
/**  */
bool CCommandingFifo::AcceptsContainer(const CCodeGenerator* pContainer)
{
	if (pContainer->DescendantOf("CDynamicSystem"))
	{
		if (pContainer->Components().DescendantsOf("CCommandingFifo"))
		{
			AddError(1700);
			return false;
		}
		else
		{
			SetContainer(pContainer);
			return true;
		}
	}
	else
	{
		AddError(101);
		return false;
	}
}
