/***************************************************************************
                          loop.cpp  -  description
                             -------------------
    begin                : Thu Feb 10 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivan_m_r@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "loop.h"

#include "function.h"
#include <strstream>
#include "rtmodule.h"
#include "dynamicsystem.h"
#include "input.h"

CLoop::CLoop()
{
}
CLoop::~CLoop()
{
}
/**  */
void CLoop::Arrange(void* ExtraParameterL = NULL)
{
	if (IllegalCIdentifier(Identifier()))
		AddError(100);
	else
	{
		SetFunctionIdentifier("loop_" + Identifier());
	  SetFunctionArgument("NULL");
		SetPThreadAttrIdentifier("attr");
		SetSchedParamIdentifier("sched_param");
		SetStartTime("gethrtime()");
	}
}
/**  */
bool CLoop::SetPeriod(long long llPeriod)
{
  strstream Stream;

  Stream << llPeriod << ends;
	CRTTask::SetPeriod(Stream.str());

	if (llPeriod > 0)
		return true;
	else
	{
		AddError(300);
		return false;
	}
}
/**  */
bool CLoop::SetName(const string& Name)
{
	SetIdentifier(Name);

	if (IllegalCIdentifier(Name))
	{
		AddError(100);
		return false;
	}	
	else
		return true;
}
/**  */
string CLoop::Class() const
{
	return "CLoop";
}
/**  */
bool CLoop::SetPriority(unsigned int PriorityL)
{
  strstream StreamL;

  StreamL << PriorityL << ends;
	CRTTask::SetPriority(StreamL.str());

	return true;
}
/**  */
bool CLoop::DescendantOf(const string& ClassNameL) const
{
	if (ClassNameL == "CLoop")
		return true;
	else
		return CRTTask::DescendantOf(ClassNameL);
}
/**  */
void CLoop::GenerateFunctions(ostream& StreamL, void* ExtraParameterL = NULL) const
{
	CStringsSet StringsSetL;

	if (Scheduling() == PERIOD)
		StreamL << "void *" << FunctionIdentifier() << "(void* param)" << endl;
	else
		StreamL << "unsigned int " << FunctionIdentifier() <<
			"(unsigned int irq, struct pt_regs *regs)" << endl;

	StreamL << "{" << endl;

	StringsSetL.clear();
	Container()->ComponentsGenerateCode(StringsSetL, this,
		"local variables declaration", ExtraParameterL);
	copy(StringsSetL.begin(), StringsSetL.end(), ostream_iterator<string>(StreamL, "\n"));

	if (Scheduling() == PERIOD)
	{
		StreamL << "  while (1)" << endl;
		StreamL << "  {" << endl;
		StreamL << "    pthread_suspend_np(pthread_self());" << endl;
		StreamL << "    do" << endl;
		StreamL << "    {" << endl;
		StreamL << "      pthread_wait_np();" << endl;
	}

	ComponentsGenerateCode(StreamL, "start time log", ExtraParameterL);
	Container()->ComponentsGenerateCode(StreamL, this, "global variables set",
		ExtraParameterL);
	ComponentsGenerateCode(StreamL, "variables set", ExtraParameterL);
	ComponentsGenerateCode(StreamL, "processes call", ExtraParameterL);
	ComponentsGenerateCode(StreamL, "variables log", ExtraParameterL);
	Container()->ComponentsGenerateCode(StreamL, this, "global variables log",
		ExtraParameterL);
	ComponentsGenerateCode(StreamL, "end time log", ExtraParameterL);

	if (Scheduling() == PERIOD)
	{
		StreamL << "    }" << endl;
		StreamL << "    while ((gethrtime() - start_time < " << Container()->Time() <<
			") && run);" << endl;
		StreamL << "  }" << endl;
	}

	StreamL << "  return 0;" << endl;
	StreamL << "}" << endl;
}
const string& CLoop::Name() const
{
	return Identifier();
}
/**  */
long long CLoop::Period() const
{
	strstream StreamL;
	long long PeriodL;

	StreamL << CRTTask::Period();
	StreamL >> PeriodL;
	return PeriodL;
}
/**  */
unsigned int CLoop::Priority() const
{
	strstream StreamL;
	long long PriorityL;

	StreamL << CRTTask::Priority();
	StreamL >> PriorityL;
	return PriorityL;
}
/**  */
void CLoop::ComponentsDependingArrange(void* ExtraParameterL = 0)
{
	CCodeGeneratorPointers OrderedComponentsL;
  CCodeGeneratorPointers::ConstIterator ComponentPI;
	CCodeGeneratorPointers::ConstIterator Component2;
	const CBlock* pBlock;
	const CInput* pInput;

	if (Container()->RepeatedValue(this, "name", Name()))
		AddError(1000);

	ComponentPI = Components().Begin();
	while (ComponentPI != Components().End())
		if ((*ComponentPI)->DescendantOf("CBlock"))
		{
			pBlock = (CBlock*)(*ComponentPI);
			if (pBlock->CandidateForOrdenation(OrderedComponentsL))
			{
				// Check redundant initialization of input variables
				for (Component2 = pBlock->Components().Begin();
					Component2 != pBlock->Components().End(); Component2++)
				if ((*Component2)->DescendantOf("CInput"))
				{
					pInput = (CInput*)(*Component2);
					if (pInput->Initialized() && pInput->ConnectedToOrderedBlock(OrderedComponentsL))
						AddError(1401, pInput);
				}

				// pBlock->CheckRedundantInitializations(OrderedComponentsL);
				OrderedComponentsL.PushBack((CCodeGenerator*)pBlock);
				ComponentPI = Components().Begin();
			}
			else
			  ComponentPI++;
		}
		else
			ComponentPI++;
 		
	ComponentPI = Components().Begin();
	while (ComponentPI != Components().End())
	{
		if ((*ComponentPI)->DescendantOf("CBlock"))
		{
			pBlock = (CBlock*)(*ComponentPI);
			if (!OrderedComponentsL.Contains(pBlock))
			{
				AddError(502, pBlock);
				OrderedComponentsL.PushBack(*ComponentPI);	
			}
    }
		else
			OrderedComponentsL.PushBack(*ComponentPI);
		ComponentPI++;
	}
	
	Components() = OrderedComponentsL;
}
/**  */
const CDynamicSystem* CLoop::Container() const
{
	return (CDynamicSystem*) CRTTask::Container();
}
/**  */
bool CLoop::RepeatedValue(const CCodeGenerator* CallerPL,
	const string& ValueNameL, const string& ValueL) const
{
	if ((CallerPL != this) && (ValueNameL == "name") &&
			(ValueL == Identifier()))
		return true;
	else
		return Components().RepeatedValue(CallerPL, ValueNameL, ValueL);
}
/**  */
void CLoop::GenerateCode(CStringsSet& StringsSetL,
	const CCodeGenerator* CallerPL, const string& SectionL,
	void* ExtraParameterL = NULL) const
{
	if (CallerPL->DescendantOf("CDynamicSystem"))
		if (SectionL == "includes")
			GenerateIncludes(StringsSetL, ExtraParameterL);
		else
			if (SectionL == "init_module variables declaration")
				GenerateInitializationVariablesDeclaration(StringsSetL, ExtraParameterL);
}
/**  */
void CLoop::GenerateCode(ostream& StreamL, const CCodeGenerator* CallerPL,
	const string& SectionL,	void* ExtraParameterL = NULL) const
{
	if (CallerPL->DescendantOf("CDynamicSystem"))
	{
		if (SectionL == "threads declaration")
			GenerateThreadDeclaration(StreamL, ExtraParameterL);
		else
			if (SectionL == "initialization functions declaration")
				GenerateInitializationFunctions(StreamL, ExtraParameterL);
			else
				if (SectionL == "initialization functions call")
					GenerateInitializationCall(StreamL, ExtraParameterL);
				else
					if (SectionL == "functions declaration")
						GenerateFunctions(StreamL, ExtraParameterL);
					else
						if (SectionL == "tasks initialization")
							GenerateInitialization(StreamL, ExtraParameterL);
						else
							if (SectionL == "tasks wakeup")
								GenerateWakeUp(StreamL, ExtraParameterL);
							else
								if (SectionL == "tasks suspend")
									GenerateSuspend(StreamL, ExtraParameterL);
								else
									if (SectionL == "tasks destruction")
										GenerateDestruction(StreamL, ExtraParameterL);
	}
  else
		if (CallerPL->DescendantOf("CCommandingFifo"))
			if (SectionL == "tasks wakeup")
				GenerateWakeUp(StreamL, ExtraParameterL);
}
/**  */
void CLoop::GenerateInitializationFunctions(ostream& StreamL,
	void* ExtraParameterL = NULL) const
{
	StreamL << "void init_" << Identifier() << "(void)" << endl;
	StreamL << "{" << endl;

	ComponentsGenerateCode(StreamL, "initialized variables declaration", ExtraParameterL);
	ComponentsGenerateCode(StreamL, "variables initialization", ExtraParameterL);
	ComponentsGenerateCode(StreamL, "initialized variables writing", ExtraParameterL);

	StreamL << "}" << endl;
}
/**  */
void CLoop::GenerateInitializationCall(ostream& StreamL, void* ExtraParameterL= NULL) const
{
  StreamL << "  init_" << Identifier() << "();" << endl;
}
/**  */
void CLoop::GenerateSuspend(ostream& StreamL, void* ExtraParameterL = NULL) const
{
	if (Scheduling() == INTERRUPT)
		StreamL << "  rtl_hard_disable_irq(" << IRQNumber() << ");" << endl;
}
/**  */
bool CLoop::SetIRQNumber(unsigned int IRQNumberL)
{
  strstream StreamL;

  StreamL << IRQNumberL << ends;
	CRTTask::SetIRQNumber(StreamL.str());
	
	return true;
}
/**  */
unsigned int CLoop::IRQNumber() const
{
	strstream StreamL;
	long long IRQNumberL;

	StreamL << CRTTask::IRQNumber();
	StreamL >> IRQNumberL;
	return IRQNumberL;
}
/**  */
bool CLoop::SetScheduling(SchedulingE Scheduling)
{
	CRTTask::SetScheduling(Scheduling);

	return true;
}
/**  */
bool CLoop::SetFloatingPoint(bool bFloatingPoint)
{
	if (bFloatingPoint)
		CRTTask::SetFloatingPoint("1");
	else
		CRTTask::SetFloatingPoint("0");

	return true;
}
/**  */
bool CLoop::FloatingPoint() const
{
	if (CRTTask::FloatingPoint() == "1")
		return true;
	else
		return false;
}
/**  */
bool CLoop::AcceptsContainer(const CCodeGenerator* pContainer)
{
	if (pContainer->DescendantOf("CDynamicSystem"))
	{
		SetContainer(pContainer);
		return true;
	}
	else
	{
		AddError(101);
		return false;
	}
}
