/***************************************************************************
                          reportingfifo.cpp  -  description
                             -------------------
    begin                : Thu Jun 1 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivan_m_r@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "reportingfifo.h"

#include <strstream>
#include "loop.h"

CReportingFifo::CReportingFifo()
{
}
CReportingFifo::~CReportingFifo()
{
}
/**  */
void CReportingFifo::Arrange(void* ExtraParameterL = NULL)
{
  strstream ConversionStreamL;

	SetMode(CRTFifo::WRITE);

	ConversionStreamL << Number() << ends;

	SetIdentifier(string("rtf") + ConversionStreamL.str() + "_" +
		Container()->Identifier() + "_time");
}
/**  */
void CReportingFifo::GenerateCode(ostream& StreamL, const CCodeGenerator* CallerPL,
	const string& SectionL, void* ExtraParameterL = NULL) const
{
	if (CallerPL->DescendantOf("CDynamicSystem"))
	{
		if (SectionL == "fifos declaration")
			GenerateDeclaration(StreamL, ExtraParameterL);
		else
			if (SectionL == "fifos initialization")
				GenerateInitialization(StreamL, ExtraParameterL);
			else
				if (SectionL == "fifos destruction")
					GenerateDestruction(StreamL, ExtraParameterL);	
	}
	else
		if (CallerPL->DescendantOf("CLoop"))
			if (SectionL == "start time log")
				GenerateStartTimeLog(StreamL, ExtraParameterL);
			else
				if (SectionL == "end time log")
					GenerateEndTimeLog(StreamL, ExtraParameterL);
}
/**  */
string CReportingFifo::Class() const
{
	return "CReportingFifo";
}
/**  */
bool CReportingFifo::DescendantOf(const string& ClassNameL) const
{
	if (ClassNameL == "CReportingFifo")
		return true;
	else
		return CRTFifo::DescendantOf(ClassNameL);
}
/**  */
void CReportingFifo::GenerateStartTimeLog(ostream& StreamL,
	void* ExtraParameterL = NULL) const
{
	// Message 0 = period's starting time
	StreamL << "      message = 0;" << endl;
	StreamL << "      write(" << Identifier() << ", &message, sizeof(message));" << endl;
	StreamL << "      time = gethrtime() - start_time;" << endl;
	StreamL << "      write(" << Identifier() << ", &time, sizeof(time));" << endl;
}
/**  */
void CReportingFifo::GenerateEndTimeLog(ostream& StreamL,
	void* ExtraParameterL = NULL) const
{
	// Message 1 = period's finishing time
	StreamL << "      message = 1;" << endl;
	StreamL << "      write(" << Identifier() << ", &message, sizeof(message));" << endl;
	StreamL << "      time = gethrtime() - start_time;" << endl;
	StreamL << "      write(" << Identifier() << ", &time, sizeof(time));" << endl;
}
/**  */
void CReportingFifo::GenerateIncludes(CStringsSet& StringsSetL,
	void* ExtraParameterL = NULL) const
{
	StringsSetL.insert("#include <unistd.h>");
	StringsSetL.insert("#include <rtl_fifo.h>");
	StringsSetL.insert("#include <rtl_time.h>");
	StringsSetL.insert("#include <sys/types.h>");
	StringsSetL.insert("#include <sys/stat.h>");
	// StringsSetL.insert("#include <fcntl.h>");		Produces a parse error in such file
}
/**  */
void CReportingFifo::GenerateCode(CStringsSet& StringsSetL,
	const CCodeGenerator* CallerPL, const string& SectionL,
	void* ExtraParameterL = NULL) const
{
	if (CallerPL->DescendantOf("CDynamicSystem"))
	{
		if (SectionL == "includes")
			GenerateIncludes(StringsSetL, ExtraParameterL);
	}
	if (CallerPL->DescendantOf("CLoop"))
		if (SectionL == "local variables declaration")
			GenerateStaticDeclarations(StringsSetL, ExtraParameterL);
}
/**  */
const CLoop* CReportingFifo::Container() const
{
	return (CLoop*)CRTFifo::Container();
}
/**  */
//Del by KDevelop: string CReportingFifo::ContainerClass() const
//Del by KDevelop: {
//Del by KDevelop: 	return "CLoop";
//Del by KDevelop: }
/**  */
void CReportingFifo::GenerateStaticDeclarations(CStringsSet& SetOfStringsL,
	void* ExtraParameterL= NULL) const
{
	SetOfStringsL.insert("  static long long message;");
	SetOfStringsL.insert("  static hrtime_t time;");
}
/**  */
//Del by KDevelop: string CReportingFifo::IncompatibleClass() const
//Del by KDevelop: {
//Del by KDevelop: 	return "CReportingFifo";
//Del by KDevelop: }
/**  */
bool CReportingFifo::SetSize(int iSize)
{
  strstream Stream;

  Stream << iSize << ends;
	CRTFifo::SetSize(Stream.str());

	if (iSize > 0)
		return true;
	else
	{
		AddError(401);
		return false;
	}
}
/**  */
int CReportingFifo::Size()
{
	strstream Stream;
	int iSize;

	Stream << CRTFifo::Size();
	Stream >> iSize;
	return iSize;
}
/**  */
bool CReportingFifo::SetNumber(unsigned int uNumber)
{
  strstream Stream;

	if (uNumber >= 0)
	{
	  Stream << uNumber << ends;
		CRTFifo::SetNumber(Stream.str());

		return true;
	}
	else
	{
		AddError(400);
		return false;
	}
}
/**  */
unsigned int CReportingFifo::Number() const
{
	strstream Stream;
	long long Number;

	Stream << CRTFifo::Number();
	Stream >> Number;
	return Number;
}
/**  */
unsigned int CReportingFifo::MaximumInComponents() const
{
	return 1;
}
/**  */
bool CReportingFifo::AcceptsContainer(const CCodeGenerator* pContainer)
{
	if (pContainer->DescendantOf("CLoop"))
	{
		if (pContainer->Components().DescendantsOf("CReportingFifo"))
		{
			AddError(1800);
			return false;
		}
		else
		{
			SetContainer(pContainer);
			return true;
		}
	}
	else
	{
		AddError(101);
		return false;
	}
}
