/***************************************************************************
                          rttaskc.cpp  -  description
                             -------------------
    begin                : Fri May 12 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivan_m_r@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "rttask.h"

#include "rtmodule.h"

CRTTask::CRTTask()
{
	m_FloatingPoint = "1";
	FunctionArgumentA = "";
	FunctionIdentifierA = "";
	FunctionParameterIdentifierA = "";
	PThreadAttrIdentifierA = "";
	PeriodA = "";
  PriorityA = "";
	SchedParamIdentifierA = "";
	StartTimeA = "";
  SchedulingA = PERIOD;
	IRQNumberA = "0";
}
CRTTask::~CRTTask()
{
}
/**  */
void CRTTask::GenerateThreadDeclaration(ostream& StreamL,
	void* ExtraParameterL = NULL) const
{
	if (Scheduling() == PERIOD)
		StreamL << "pthread_t " << Identifier() << ";" << endl;
}
/**  */
void CRTTask::GenerateInitialization(ostream& StreamL, void* ExtraParameterL = NULL) const
{
	if (Scheduling() == PERIOD)
	{
	  StreamL << "  pthread_attr_init(&" << PThreadAttrIdentifier() << ");" << endl;
		StreamL << "  " << SchedParamIdentifier() << ".sched_priority = " << Priority() <<
			";" << endl;
		StreamL << "  pthread_attr_setschedparam(&" << PThreadAttrIdentifier() << ", &" <<
			SchedParamIdentifier() << ");" << endl;
		StreamL << "  pthread_create(&" << Identifier() << ", &" << PThreadAttrIdentifier() <<
			", " << FunctionIdentifier() << ", (void *)" << FunctionArgument() << ");" << endl;
	  StreamL << "  pthread_setfp_np(" << Identifier() << ", " << FloatingPoint() << ");" <<
			endl;
		StreamL << "  pthread_make_periodic_np(" << Identifier() << ", " << StartTime() <<
			", " << Period() << ");" << endl;
	}
	else
	{
		StreamL << "  rtl_stop_interrupts();" << endl;
		StreamL << "  rtl_request_irq(" << IRQNumber() << ", &" << FunctionIdentifier() <<
			");" << endl;
		StreamL << "  rtl_allow_interrupts();" << endl;
	}
}
/**  */
void CRTTask::SetPThreadAttrIdentifier(const string& IdentifierL)
{
	PThreadAttrIdentifierA = IdentifierL;
}
/**  */
const string& CRTTask::PThreadAttrIdentifier() const
{
	return PThreadAttrIdentifierA;
}
/**  */
void CRTTask::SetStartTime(const string& StartTimeL)
{
	StartTimeA = StartTimeL;
}
/**  */
const string& CRTTask::StartTime() const
{
	return StartTimeA;
}
/**  */
void CRTTask::SetSchedParamIdentifier(const string& IdentifierL)
{
	SchedParamIdentifierA = IdentifierL;
}
/**  */
const string& CRTTask::SchedParamIdentifier() const
{
	return SchedParamIdentifierA;
}
/**  */
void CRTTask::SetPriority(const string& PriorityL)
{
	PriorityA = PriorityL;
}
/**  */
const string& CRTTask::Priority() const
{
	return PriorityA;
}
/**  */
void CRTTask::SetFunctionIdentifier(const string& IdentifierL)
{
	FunctionIdentifierA = IdentifierL;
}
/**  */
const string& CRTTask::FunctionIdentifier() const
{
	return FunctionIdentifierA;
}
/**  */
void CRTTask::SetFunctionArgument(const string& ArgumentL)
{
	FunctionArgumentA = ArgumentL;
}
/**  */
const string& CRTTask::FunctionArgument() const
{
	return FunctionArgumentA;
}
/**  */
void CRTTask::SetPeriod(const string& PeriodL)
{
	PeriodA = PeriodL;
}
/**  */
const string& CRTTask::Period() const
{
	return PeriodA;
}
/**  */
//Del by KDevelop: void CRTTask::Arrange(void* ExtraParameterL = NULL)
//Del by KDevelop: {
//Del by KDevelop: 	if (IllegalCIdentifier(Identifier()))
//Del by KDevelop: 		ErrorsReport().insert("\"" + Identifier() + "\" is not a valid C identifier");
//Del by KDevelop: /*
//Del by KDevelop: 	else
//Del by KDevelop: 	{
//Del by KDevelop: 		SetPThreadAttrIdentifier(Container()->PThreadAttrIdentifier());
//Del by KDevelop: 		SetSchedParamIdentifier(Container()->SchedParamIdentifier());
//Del by KDevelop: 	}
//*/
//}
/**  */
void CRTTask::SetFunctionParameterIdentifier(const string& IdentifierL)
{
	FunctionParameterIdentifierA = IdentifierL;
}
/**  */
const string& CRTTask::FunctionParameterIdentifier() const
{
	return FunctionParameterIdentifierA;
}
/**  */
//Del by KDevelop: string CRTTask::Class() const
//Del by KDevelop: {
//Del by KDevelop: 	return "CRTTask";
//Del by KDevelop: }
/**  */
void CRTTask::SetFloatingPoint(const string& FloatingPoint)
{
	m_FloatingPoint = FloatingPoint;
}
/**  */
const string& CRTTask::FloatingPoint() const
{
	return m_FloatingPoint;
}
/**  */
bool CRTTask::DescendantOf(const string& ClassNameL) const
{
	if (ClassNameL == "CRTTask")
		return true;
	else
		return CCodeGenerator::DescendantOf(ClassNameL);
}
/**  */
void CRTTask::GenerateIncludes(CStringsSet& StringsSetL,
	void* ExtraParameterL = NULL) const
{
	if (Scheduling() == PERIOD)
	{
		StringsSetL.insert("#include <pthread.h>");
		StringsSetL.insert("#include <rtl_sched.h>");
	}
	else
		StringsSetL.insert("#include <rtl_sync.h>");
}
/**  */
void CRTTask::GenerateDestruction(ostream& StreamL, void* ExtraParameterL = NULL) const
{
	if (Scheduling() == PERIOD)
		StreamL << "  pthread_delete_np(" << Identifier() << ");" << endl;
}
/**  */
const CRTModule* CRTTask::Container() const
{
	return (CRTModule*)CCodeGenerator::Container();
}
/**  */
string CRTTask::ContainerClass() const
{
	return "CRTModule";
}
/**  */
void CRTTask::GenerateWakeUp(ostream& StreamL, void* ExtraParameterL = NULL) const
{
	if (Scheduling() == PERIOD)
		StreamL << "  pthread_wakeup_np(" << Identifier() << ");" << endl;
	else
		StreamL << "  rtl_hard_enable_irq(" << IRQNumber() << ");" << endl;
}
/**  */
void CRTTask::SetScheduling(SchedulingE SchedulingL)
{
	SchedulingA = SchedulingL;
}
/**  */
void CRTTask::SetIRQNumber(const string& IRQNumberL)
{
	IRQNumberA = IRQNumberL;
}
/**  */
const string& CRTTask::IRQNumber() const
{
  return IRQNumberA;
}
/**  */
enum CRTTask::SchedulingE CRTTask::Scheduling() const
{
	return SchedulingA;
}
/**  */
void CRTTask::GenerateSuspend(ostream& StreamL, void* ExtraParameterL = NULL) const
{
	if (Scheduling() == PERIOD)
		StreamL << "  pthread_suspend_np(" << Identifier() << ");" << endl;
	else
		StreamL << "  rtl_hard_disable_irq(" << IRQNumber() << ");" << endl;
}
/**  */
void CRTTask::GenerateInitializationVariablesDeclaration(CStringsSet& StringsSetL,
	void* ExtraParameterL = NULL) const
{
	if (Scheduling() == PERIOD)
	{
	  StringsSetL.insert("  pthread_attr_t " + PThreadAttrIdentifier() + ";");
	  StringsSetL.insert("  struct sched_param " + SchedParamIdentifier() + ";");
	}
}
