/***************************************************************************
                          rttask.h  -  description
                             -------------------
    begin                : Fri May 12 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivan_m_r@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef RTTASK_H
#define RTTASK_H

#include "codegenerator.h"

/**
  *@author Ivan Martinez
  */

class CRTModule;

class CRTTask : public CCodeGenerator {
public:
	enum SchedulingE {PERIOD, INTERRUPT};
	CRTTask();
	~CRTTask();
  /**  */
  const CRTModule* Container() const;
  /**  */
  virtual string ContainerClass() const;
  /**  */
  virtual bool DescendantOf(const string&) const;
  /**  */
  const string& FloatingPoint() const;
  /**  */
  const string& FunctionArgument() const;
  /**  */
  const string& FunctionIdentifier() const;
  /**  */
  const string& FunctionParameterIdentifier() const;
  /**  */
  const string& IRQNumber() const;
  /**  */
  const string& Period() const;
  /**  */
  const string& Priority() const;
  /**  */
  const string& PThreadAttrIdentifier() const;
  /**  */
  const string& SchedParamIdentifier() const;
  /**  */
  enum SchedulingE Scheduling() const;
  /**  */
	const string& StartTime() const;
protected:
  /**  */
  virtual void GenerateDestruction(ostream&, void* = NULL) const;
  /**  */
  virtual void GenerateIncludes(CStringsSet&, void* = NULL) const;
  /**  */
  virtual void GenerateInitialization(ostream&, void* = NULL) const;
  /**  */
  virtual void GenerateInitializationVariablesDeclaration(CStringsSet&,
		void* = NULL) const;
  /**  */
  virtual void GenerateSuspend(ostream&, void* = NULL) const;
  /**  */
  virtual void GenerateThreadDeclaration(ostream&, void* = NULL) const;
  /**  */
  virtual void GenerateWakeUp(ostream&, void* = NULL) const;
  /**  */
  void SetFloatingPoint(const string&);
  /**  */
  void SetFunctionArgument(const string&);
  /**  */
  void SetFunctionIdentifier(const string&);
  /**  */
  void SetFunctionParameterIdentifier(const string&);
  /**  */
  void SetIRQNumber(const string&);
  /**  */
  void SetPeriod(const string&);
  /**  */
  void SetPriority(const string&);
  /**  */
  void SetPThreadAttrIdentifier(const string&);
  /**  */
  void SetSchedParamIdentifier(const string&);
  /**  */
  void SetScheduling(SchedulingE);
  /**  */
	void SetStartTime(const string& StartTimeL);
private: // Private attributes
  /**  */
  string PThreadAttrIdentifierA;
  /**  */
  string SchedParamIdentifierA;
  /**  */
  string PriorityA;
  /**  */
  string FunctionIdentifierA;
  /**  */
  string FunctionArgumentA;
  /**  */
  string PeriodA;
  /**  */
  string StartTimeA;
  /**  */
  string FunctionParameterIdentifierA;
  /**  */
  SchedulingE SchedulingA;
  /**  */
  string m_FloatingPoint;
  /**  */
  string IRQNumberA;
};

#endif
