/***************************************************************************
                          settingfifo.cpp  -  description
                             -------------------
    begin                : Thu Jun 1 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivan_m_r@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "settingfifo.h"

#include "initializablevariable.h"
#include <strstream>
#include "systemvariable.h"

CSettingFifo::CSettingFifo()
{
}
CSettingFifo::~CSettingFifo()
{
}
/**  */
void CSettingFifo::Arrange(void* ExtraParameterL = NULL)
{
  strstream ConversionStreamL;

	SetMode(CRTFifo::READ);

	ConversionStreamL << Number() << ends;

	SetIdentifier(string("rtf") + ConversionStreamL.str() + "_" +
		Container()->LocalIdentifier());
	SetBuffer("&" + Container()->Identifier());
	SetBufferSize("sizeof(" + Container()->Identifier() + ")");
}
/**  */
string CSettingFifo::Class() const
{
	return "CSettingFifo";
}
/**  */
bool CSettingFifo::DescendantOf(const string& ClassNameL) const
{
	if (ClassNameL == "CSettingFifo")
		return true;
	else
		return CVariableFifo::DescendantOf(ClassNameL);
}
/**  */
const CInitializableVariable* CSettingFifo::Container() const
{
	return (CInitializableVariable*)CRTFifo::Container();
}
/**  */
//Del by KDevelop: string CSettingFifo::ContainerClass() const
//Del by KDevelop: {
//Del by KDevelop: 	return "CInitializableVariable";
//Del by KDevelop: }
/**  */
string CSettingFifo::IncompatibleClass() const
{
	return "CSettingFifo";
}
/**  */
void CSettingFifo::GenerateCode(ostream& StreamL, const CCodeGenerator* CallerPL,
	const string& SectionL, void* ExtraParameterL = NULL) const
{
	if (CallerPL->DescendantOf("CDynamicSystem"))
	{
		if (SectionL == "fifos declaration")
			GenerateDeclaration(StreamL, ExtraParameterL);
		else
			if (SectionL == "fifos initialization")
				GenerateInitialization(StreamL, ExtraParameterL);
			else
				if (SectionL == "fifos destruction")
					GenerateDestruction(StreamL, ExtraParameterL);	
	}
	else
		if (CallerPL->DescendantOf("CLoop"))
			if (SectionL == "variables set")
				GenerateFifoReading(StreamL, ExtraParameterL);
			else
				if (SectionL == "global variables set")
					GenerateLockedFifoReading(StreamL, ExtraParameterL);
}
/**  */
void CSettingFifo::GenerateLockedFifoReading(ostream& StreamL,
	void* ExtraParameterL = NULL) const
{
	if (Container()->DescendantOf("CSystemVariable"))
	{
		StreamL << "      rtl_spin_lock_irqsave(&" <<
			((CSystemVariable*)Container())->LockIdentifier() << ", flags);" << endl;
		StreamL << "      read(" << Identifier() << ", " << Buffer() << ", " << BufferSize() <<
			");" << endl;
		StreamL << "      rtl_spin_unlock_irqrestore(&" <<
			((CSystemVariable*)Container())->LockIdentifier() << ", flags);" << endl;
	}
}
/**  */
bool CSettingFifo::SetNumber(unsigned int uNumber)
{
  strstream Stream;

  Stream << uNumber << ends;
	CVariableFifo::SetNumber(Stream.str());

	if (uNumber >= 0)
	{
		return true;
	}
	else
	{
		AddError(400);
		return false;
	}
}
/**  */
unsigned int CSettingFifo::Number() const
{
	strstream Stream;
	long long Number;

	Stream << CVariableFifo::Number();
	Stream >> Number;
	return Number;
}
/**  */
bool CSettingFifo::SetSize(int iSize)
{
  strstream Stream;

  Stream << iSize << ends;
	CRTFifo::SetSize(Stream.str());

	if (iSize > 0)
		return true;
	else
	{
		AddError(401);
		return false;
	}
}
/**  */
int CSettingFifo::Size()
{
	strstream Stream;
	int iSize;

	Stream << CRTFifo::Size();
	Stream >> iSize;
	return iSize;
}
/**  */
unsigned int CSettingFifo::MaximumInComponents() const
{
	return 1;
}
/**  */
bool CSettingFifo::AcceptsContainer(const CCodeGenerator* pContainer)
{
	if (pContainer->DescendantOf("CInitializableVariable"))
	{
		if (pContainer->Components().DescendantsOf("CSettingFifo"))
		{
			AddError(2000);
			return false;
		}
		else
		{
			SetContainer(pContainer);
			return true;
		}
	}
	else
	{
		AddError(101);
		return false;
	}
}
