/***************************************************************************
                          systemvariable.cpp  -  description
                             -------------------
    begin                : Tue Apr 18 2000
    copyright            : (C) 2000 by 
    email                : ivan_m_r@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "systemvariable.h"
#include "dynamicsystem.h"

CSystemVariable::CSystemVariable()
{
}
CSystemVariable::~CSystemVariable()
{
}
/**  */
string CSystemVariable::Class() const
{
	return "CSystemVariable";
}
/**  */
bool CSystemVariable::SetName(const string& NameL)
{
	SetLocalIdentifier(NameL);

	if (IllegalCIdentifier(NameL))
	{
		AddError(600);
		return false;
	}	
	else
	{
		SetIdentifier("systemvar_" + LocalIdentifier());
		SetLockIdentifier("lock_" + LocalIdentifier());
		return true;
	}
}
/**  */
bool CSystemVariable::SetType(const string& TypeL)
{
	CVariable::SetType(TypeL);

	if (IllegalCIdentifier(TypeL))
	{
		AddError(602);
		return false;
	}
	else
		return true;
}
/**  */
bool CSystemVariable::DescendantOf(const string& ClassNameL) const
{
	if (ClassNameL == "CSystemVariable")
		return true;
	else
		return CInitializableVariable::DescendantOf(ClassNameL);
}
/**  */
bool CSystemVariable::SetInitialization(const string& InitializationL)
{
	CInitializableVariable::SetInitialization(InitializationL);

	return true;
}
/**  */
//Del by KDevelop: const string& CSystemVariable::Initialization() const
//Del by KDevelop: {
//Del by KDevelop: 	return CInitializableVariable::Initialization();
//Del by KDevelop: }
/**  */
void CSystemVariable::GenerateGlobalReading(ostream& StreamL,
	void* ExtraParameterL = NULL) const
{
	StreamL << "  rtl_spin_lock_irqsave(&" << LockIdentifier() << ", flags);" << endl;
	StreamL << "  " << LocalIdentifier() << " = " << Identifier() << ";" << endl;
	StreamL << "  rtl_spin_unlock_irqrestore(&" << LockIdentifier() << ", flags);" << endl;
}
/**  */
void CSystemVariable::GenerateGlobalWriting(ostream& StreamL,
	void* ExtraParameterL = NULL) const
{
	StreamL << "  rtl_spin_lock_irqsave(&" << LockIdentifier() << ", flags);" << endl;
	StreamL << "  " << Identifier() << " = " << LocalIdentifier() << ";" << endl;
	StreamL << "  rtl_spin_unlock_irqrestore(&" << LockIdentifier() << ", flags);" << endl;
}
/**  */
void CSystemVariable::GenerateCode(ostream& StreamL, const CCodeGenerator* CallerPL,
	const string& SectionL, void* ExtraParameterL = NULL) const
{
	if (CallerPL->DescendantOf("CDynamicSystem"))
	{
		if (SectionL == "global variables declaration")
			GenerateGlobalDeclarations(StreamL, ExtraParameterL);
		else
			if (SectionL == "initialized variables declaration")
				GenerateInitializationDeclaration(StreamL, ExtraParameterL);
			else
				if (SectionL == "variables initialization")
					GenerateInitialization(StreamL, ExtraParameterL);
				else
					if (SectionL == "initialized variables writing")
						GenerateInitializationWriting(StreamL, ExtraParameterL);
	}
	else
		if (CallerPL->DescendantOf("CBlock"))
			if (SectionL == "local system variable declarations")
				GenerateLocalDeclarations(StreamL, ExtraParameterL);
			else			
				if (SectionL == "system variables reading")
					GenerateGlobalReading(StreamL, ExtraParameterL);
				else
					if (SectionL == "system variables writing")
						GenerateGlobalWriting(StreamL, ExtraParameterL);
}
/**  */
const CDynamicSystem* CSystemVariable::Container() const
{
	return (CDynamicSystem*)CCodeGenerator::Container();
}
/**  */
void CSystemVariable::GenerateGlobalDeclarations(ostream& StreamL,
	void* ExtraParameterL = NULL) const
{
	StreamL << "spinlock_t " << LockIdentifier() << " = SPIN_LOCK_UNLOCKED;" << endl;
	StreamL << Type() << " " << Identifier() << ";" << endl;
}
/**  */
void CSystemVariable::SetLockIdentifier(const string& LockIdentifierL)
{
	LockIdentifierA = LockIdentifierL;
}
/**  */
const string& CSystemVariable::LockIdentifier() const
{
	return LockIdentifierA;
}
/**  */
void CSystemVariable::GenerateCode(CStringsSet& StringsSetL,
	const CCodeGenerator* CallerPL, const string& SectionL,
	void* ExtraParameterL= NULL) const
{
	if (CallerPL->DescendantOf("CDynamicSystem"))
	{
		if (SectionL == "includes")
			GenerateIncludes(StringsSetL, ExtraParameterL);
/*
		else
			if (SectionL == "global variables declaration")
				GenerateGlobalDeclarations(StringsSetL, ExtraParameterL);
*/
  }
	else
		if (CallerPL->DescendantOf("CLoop"))
		{
			if (SectionL == "local variables declaration")
				GenerateLocalDeclarations(StringsSetL, ExtraParameterL);
		}
		else
			if (CallerPL->DescendantOf("CBlock"))
				if (SectionL == "local variables declaration")
					GenerateLocalDeclarations(StringsSetL, ExtraParameterL);
}
/**  */
void CSystemVariable::GenerateIncludes(CStringsSet& StringsSetL,
	void* ExtraParameterL= NULL) const
{
	StringsSetL.insert("#include <rtl_sync.h>");
}
/**  */
void CSystemVariable::GenerateLocalDeclarations(CStringsSet& StringsSet,
	void* pExtraParameter = NULL) const
{
	StringsSet.insert("  static unsigned long flags;");
}
/**  */
void CSystemVariable::GenerateLocalDeclarations(ostream& Stream,
	void* pExtraParameter = NULL) const
{
	CVariable::GenerateLocalDeclarations(Stream, pExtraParameter);
}
/**  */
bool CSystemVariable::AcceptsContainer(const CCodeGenerator* pContainer)
{
	if (pContainer->DescendantOf("CDynamicSystem"))
	{
		SetContainer(pContainer);
		return true;
	}
	else
	{
		AddError(101);
		return false;
	}
}
/**  */
const string& CSystemVariable::Name() const
{
	return LocalIdentifier();
}
